@@ -1,42 +1,69 @@
-#' Applies a fast-and-frugal tree to a dataset and generates several accuracy statistics
+#' Apply an FFT to data and generate accuracy statistics
 #'
-#' @param x FFTrees.
-#' @param mydata dataframe.
-#' @param newdata dataframe.
-#' @param allNA.pred logical. What should be predicted if all cue values in tree are NA? Default is FALSE
-#' @importFrom testthat expect_true
-#' @return A list of length 4 containing
+#' @description \code{fftrees_apply} applies a fast-and-frugal tree (FFT, as an \code{FFTrees} object)
+#' to a dataset and generates corresponding accuracy statistics.
+#'
+#' \code{fftrees_apply} is called internally by the main \code{\link{FFTrees}} function
+#' (with \code{mydata = "train"} and --- if test data exists --- \code{mydata = "test"})
+#' and when predicting outcomes for new data by \code{\link{predict.FFTrees}}.
+#'
+#' @param x An FFT to apply (as an \code{FFTrees} object).
+#' @param mydata character. Data type to which FFT should be applied (either \code{"train"} or \code{"test"}).
+#' @param newdata dataframe. New data to which FFT should be applied.
+#' @param allNA.pred logical. What should be predicted if all cue values in tree are NA? Default is \code{FALSE}.
+#'
+#' @return A modified \code{FFTrees} object (with lists in \code{x$trees} containing information on FFT decisions and statistics).
 #'
-#' @export
 #' @keywords internal
 #'
+#' @seealso
+#' \code{\link{FFTrees}} for creating FFTs from and applying them to data.
 #'
+#' @importFrom testthat expect_true
+#' @importFrom tibble as_tibble tibble
+#'
+#' @export
 
 fftrees_apply <- function(x,
                           mydata = NULL,
                           newdata = NULL,
                           allNA.pred = FALSE) {
+
+  # Prepare: ------
+
   testthat::expect_true(mydata %in% c("train", "test"))
 
+  # Get data (corresponding to mydata and newdata):
+
   if (mydata == "train") {
+
     data <- x$data$train
+
   } else if (mydata == "test") {
+
     if (is.null(newdata)) {
+
       testthat::expect_true(!is.null(x$data$test))
-    } else {
-      x$data$test <- newdata
+
+      } else {
+
+        x$data$test <- newdata  # replaces existing test data in x by newdata!
     }
 
+    valid_train_test_data(train_data = x$data$train, test_data = x$data$test)  # verify (without consequences)
+
     data <- x$data$test
+
   }
 
   criterion_v <- data[[x$criterion_name]]
   criterion_n <- length(criterion_v)
 
-  # Setup outputs
 
-  #  [decisions_ls]
-  #    A list containing dataframes with one row per case, and one column per tree
+  # Setup outputs: ------
+
+  #  1. [decisions_ls]: ----
+  #     A list containing tibbles with one row per case, and one column per tree:
 
   decisions_ls <- lapply(1:x$trees$n, FUN = function(i) {
     tibble::tibble(
@@ -52,13 +79,15 @@ fftrees_apply <- function(x,
 
   names(decisions_ls) <- paste0("tree_", 1:x$trees$n)
 
-  # [level_stats_ls]
-  #   A list with one element per tree, each containing cumulative level statistics
+  # 2. [level_stats_ls]: ----
+  #    A list with one element per tree, each containing cumulative level statistics:
 
   level_stats_ls <- vector("list", length = x$trees$n)
 
-  # LOOP
-  #  Loop over trees
+
+  # LOOPs: ------
+
+  #  Loop over trees: ----
 
   for (tree_i in 1:x$trees$n) {
 
@@ -72,7 +101,6 @@ fftrees_apply <- function(x,
 
     decisions_df <- decisions_ls[[tree_i]]
 
-
     costc.level <- sapply(cue_v, FUN = function(cue_i) {
       if (cue_i %in% names(x$params$cost.cues)) {
         cost.cue_i <- x$params$cost.cues[[cue_i]]
@@ -88,8 +116,7 @@ fftrees_apply <- function(x,
       cue_cost_cum = costc.level.cum
     )
 
-
-    # level_stats_i contains cumulative level statistics
+    # level_stats_i contains cumulative level statistics:
     level_stats_i <- data.frame(
       tree = tree_i,
       level = 1:level_n,
@@ -103,14 +130,14 @@ fftrees_apply <- function(x,
 
     critical_stats_v <- c("n", "hi", "fa", "mi", "cr", "sens", "spec", "far", "ppv", "npv", "acc", "bacc", "wacc", "cost_decisions")
 
-    # Add stat names to level_stats_i
+    # Add stat names to level_stats_i:
     level_stats_i[critical_stats_v] <- NA
     level_stats_i$cost_cues <- NA
 
-    # Loop over levels
+    # Loop over levels: ----
     for (level_i in 1:level_n) {
 
-      # Get definitions for current level
+      # Get definitions for current level:
       cue_i <- cue_v[level_i]
       class_i <- class_v[level_i]
       direction_i <- direction_v[level_i]
@@ -121,7 +148,6 @@ fftrees_apply <- function(x,
 
       decisions_df$current_cue_values <- cue_values
 
-
       if (is.character(threshold_i)) {
         threshold_i <- unlist(strsplit(threshold_i, ","))
       }
@@ -159,21 +185,20 @@ fftrees_apply <- function(x,
         classify.now <- is.na(decisions_df$decision)
       }
 
-      # Convert NAs
+      # Convert NAs: ----
 
-      # If it is not the final node, then don't classify NA cases
+      # If it is not the final node, then don't classify NA cases:
       if (exit_i %in% c(0, 1)) {
         classify.now[is.na(classify.now)] <- FALSE
       }
 
-
-      # If it is the final node, then classify NA cases according to most common class
+      # If it is the final node, then classify NA cases according to most common class:
       if (exit_i %in% .5) {
         decisions_df$current_decision[is.na(decisions_df$current_decision)] <- allNA.pred
       }
 
 
-      # Define critical values for current decisions
+      # Define critical values for current decisions: ----
 
       decisions_df$decision[classify.now] <- decisions_df$current_decision[classify.now]
       decisions_df$levelout[classify.now] <- level_i
@@ -184,10 +209,9 @@ fftrees_apply <- function(x,
       decisions_df$cost_decision[decisions_df$criterion == FALSE & decisions_df$decision == TRUE] <- x$params$cost.outcomes$fa
       decisions_df$cost_decision[decisions_df$criterion == FALSE & decisions_df$decision == FALSE] <- x$params$cost.outcomes$cr
 
-
       decisions_df$cost <- decisions_df$cost_cue + decisions_df$cost_decision
 
-      # Get cumulative level stats
+      # Get cumulative level stats: ----
 
       my_level_stats_i <- classtable(
         prediction_v = decisions_df$decision[!is.na(decisions_df$decision)],
@@ -197,32 +221,32 @@ fftrees_apply <- function(x,
         cost.outcomes = x$params$cost.outcomes
       )
 
-
       # level_stats_i$costc <- sum(cost_cues[,tree_i], na.rm = TRUE)
       level_stats_i[level_i, critical_stats_v] <- my_level_stats_i[, critical_stats_v]
 
-      # Add cue cost and cost
+
+      # Add cue cost and cost: ----
 
       level_stats_i$cost_cues[level_i] <- mean(decisions_df$cost_cue[!is.na(decisions_df$decision)])
       level_stats_i$cost[level_i] <- level_stats_i$cost_cues[level_i] + level_stats_i$cost_decisions[level_i]
     }
 
-    # Add final tree results to level_stats_ls and decisions_ls
+    # Add final tree results to level_stats_ls and decisions_ls: ----
 
     level_stats_ls[[tree_i]] <- level_stats_i
 
     decisions_ls[[tree_i]] <- decisions_df[, names(decisions_df) %in% c("current_decision", "current_cue_values") == FALSE]
   }
 
-  # Aggregate results
+  # Aggregate results: ----
   {
 
-    # Combine all levelstats into one dataframe
+    # Combine all levelstats into one dataframe:
     level_stats <- do.call("rbind", args = level_stats_ls)
 
-    # [tree_stats]
-    #  One row per tree definitions and statistics
-    # CUMULATIVE TREE STATS
+    # 3. [tree_stats]: ----
+    #  One row per tree definitions and statistics:
+    # CUMULATIVE TREE STATS:
 
     helper <- paste(level_stats$tree, level_stats$level, sep = ".")
     maxlevs <- paste(rownames(tapply(level_stats$level, level_stats$tree, FUN = which.max)), tapply(level_stats$level, level_stats$tree, FUN = which.max), sep = ".")
@@ -231,7 +255,7 @@ fftrees_apply <- function(x,
     rownames(tree_stats) <- 1:nrow(tree_stats)
 
 
-    # Calculate pci and mcu
+    # Calculate pci and mcu: ----
 
     for (tree_i in 1:x$trees$n) {
       max.lookups <- nrow(data) * length(x$cue_names)
@@ -242,11 +266,26 @@ fftrees_apply <- function(x,
     }
   }
 
-  # Add results to x -------------------------
+  # Add results to trees in x: ------
 
-  x$trees$stats[[mydata]] <- tree_stats
-  x$trees$level_stats[[mydata]] <- level_stats
-  x$trees$decisions[[mydata]] <- decisions_ls
+  x$trees$stats[[mydata]]       <- tibble::as_tibble(tree_stats)
+  x$trees$level_stats[[mydata]] <- tibble::as_tibble(level_stats)
+  x$trees$decisions[[mydata]]   <- decisions_ls
+
+  # Best tree:
+  if (mydata == "train"){
+    x$trees$best$train <- select_best_tree(x, data = mydata, goal = x$params$goal)
+  } else if (mydata == "test"){
+    x$trees$best$test <- select_best_tree(x, data = mydata, goal = x$params$goal)
+  } else {
+    stop("mydata is neither 'train' nor 'test'")
+  }
+
+
+  # Output: ------
 
   return(x)
-}
+
+} # fftrees_apply().
+
+# eof.
