@@ -1,10 +1,19 @@
 #' Fit competitive algorithms
 #'
-#' @param x FFTrees.
+#' @description \code{fftrees_fitcomp} fits competitive algorithms for binary classification tasks
+#' (e.g., LR, CART, RF, SVM) to the data and parameters specified in an \code{FFTrees} object.
 #'
+#' \code{fftrees_fitcomp} is called by the main \code{\link{FFTrees}} function
+#' when creating FFTs from and applying them to data (unless \code{do.comp = FALSE}).
 #'
+#' @param x An \code{FFTrees} object.
 #'
+#' @seealso
+#' \code{\link{FFTrees}} for creating FFTs from and applying them to data.
+
 fftrees_fitcomp <- function(x) {
+
+  # Parameters: ------
   do.lr <- x$params$do.lr
   do.svm <- x$params$do.svm
   do.cart <- x$params$do.cart
@@ -17,14 +26,13 @@ fftrees_fitcomp <- function(x) {
     do.rf <- FALSE
   }
 
-
   my_cols <- c(
     "algorithm", "n", "hi", "fa", "mi", "cr", "sens", "spec", "far",
     "ppv", "npv", "acc", "bacc", "cost", "cost_decisions", "cost_cues"
   )
 
 
-  # FFTrees
+  # FFTrees: ----
 
   x$competition$train <- x$trees$stats$train %>%
     dplyr::filter(tree == 1) %>%
@@ -45,7 +53,9 @@ fftrees_fitcomp <- function(x) {
     }
   }
 
-  # LR
+
+  # LR: ----
+
   {
     if (do.lr) {
       lr.acc <- comp.pred(
@@ -63,19 +73,21 @@ fftrees_fitcomp <- function(x) {
 
       x$competition$train <- x$competition$train %>%
         dplyr::bind_rows(lr.stats[["train"]] %>%
-          dplyr::mutate(algorithm = "lr") %>%
-          dplyr::mutate(cost_cues = NA) %>%
-          dplyr::select(tidyselect::all_of(my_cols)))
+                           dplyr::mutate(algorithm = "lr") %>%
+                           dplyr::mutate(cost_cues = NA) %>%
+                           dplyr::select(tidyselect::all_of(my_cols)))
 
       x$competition$test <- x$competition$test %>%
         dplyr::bind_rows(lr.stats[["test"]] %>%
-          dplyr::mutate(algorithm = "lr") %>%
-          dplyr::mutate(cost_cues = NA) %>%
-          dplyr::select(tidyselect::all_of(my_cols)))
+                           dplyr::mutate(algorithm = "lr") %>%
+                           dplyr::mutate(cost_cues = NA) %>%
+                           dplyr::select(tidyselect::all_of(my_cols)))
     }
   }
 
-  # CART
+
+  # CART: ----
+
   {
     if (do.cart) {
       cart.acc <- comp.pred(
@@ -93,19 +105,21 @@ fftrees_fitcomp <- function(x) {
 
       x$competition$train <- x$competition$train %>%
         dplyr::bind_rows(cart.stats[["train"]] %>%
-          dplyr::mutate(algorithm = "cart") %>%
-          dplyr::mutate(cost_cues = NA) %>%
-          dplyr::select(tidyselect::all_of(my_cols)))
+                           dplyr::mutate(algorithm = "cart") %>%
+                           dplyr::mutate(cost_cues = NA) %>%
+                           dplyr::select(tidyselect::all_of(my_cols)))
 
       x$competition$test <- x$competition$test %>%
         dplyr::bind_rows(cart.stats[["test"]] %>%
-          dplyr::mutate(algorithm = "cart") %>%
-          dplyr::mutate(cost_cues = NA) %>%
-          dplyr::select(tidyselect::all_of(my_cols)))
+                           dplyr::mutate(algorithm = "cart") %>%
+                           dplyr::mutate(cost_cues = NA) %>%
+                           dplyr::select(tidyselect::all_of(my_cols)))
     }
   }
 
-  # rf
+
+  # RF: ----
+
   {
     if (do.rf) {
       rf.acc <- comp.pred(
@@ -124,19 +138,21 @@ fftrees_fitcomp <- function(x) {
 
       x$competition$train <- x$competition$train %>%
         dplyr::bind_rows(rf.stats[["train"]] %>%
-          dplyr::mutate(algorithm = "rf") %>%
-          dplyr::mutate(cost_cues = NA) %>%
-          dplyr::select(tidyselect::all_of(my_cols)))
+                           dplyr::mutate(algorithm = "rf") %>%
+                           dplyr::mutate(cost_cues = NA) %>%
+                           dplyr::select(tidyselect::all_of(my_cols)))
 
       x$competition$test <- x$competition$test %>%
         dplyr::bind_rows(rf.stats[["test"]] %>%
-          dplyr::mutate(algorithm = "rf") %>%
-          dplyr::mutate(cost_cues = NA) %>%
-          dplyr::select(tidyselect::all_of(my_cols)))
+                           dplyr::mutate(algorithm = "rf") %>%
+                           dplyr::mutate(cost_cues = NA) %>%
+                           dplyr::select(tidyselect::all_of(my_cols)))
     }
   }
 
-  # svm
+
+  # SVM: ----
+
   {
     if (do.svm) {
       svm.acc <- comp.pred(
@@ -147,7 +163,6 @@ fftrees_fitcomp <- function(x) {
         model = NULL
       )
 
-
       svm.stats <- svm.acc$accuracy
       svm.model <- svm.acc$model
 
@@ -155,18 +170,23 @@ fftrees_fitcomp <- function(x) {
 
       x$competition$train <- x$competition$train %>%
         dplyr::bind_rows(svm.stats[["train"]] %>%
-          dplyr::mutate(algorithm = "svm") %>%
-          dplyr::mutate(cost_cues = NA) %>%
-          dplyr::select(tidyselect::all_of(my_cols)))
+                           dplyr::mutate(algorithm = "svm") %>%
+                           dplyr::mutate(cost_cues = NA) %>%
+                           dplyr::select(tidyselect::all_of(my_cols)))
 
       x$competition$test <- x$competition$test %>%
         dplyr::bind_rows(svm.stats[["test"]] %>%
-          dplyr::mutate(algorithm = "svm") %>%
-          dplyr::mutate(cost_cues = NA) %>%
-          dplyr::select(tidyselect::all_of(my_cols)))
+                           dplyr::mutate(algorithm = "svm") %>%
+                           dplyr::mutate(cost_cues = NA) %>%
+                           dplyr::select(tidyselect::all_of(my_cols)))
     }
   }
 
 
+  # Output: ------
+
   return(x)
-}
+
+} # fftrees_fitcomp().
+
+# eof.
